<?
require_once("conf/dokuwiki.php");
require_once("inc/common.php");

/**
 * Link Formaters
 *
 * Each of these functions need to set
 *
 * $link   URL to use in href=""
 * $name   HTML to enclose in <a> with proper special char encoding
 * $class  CSS class to set on link
 * $target which target to use (blank) for current window
 * $style  Additonal style attribute set with style=""
 * $pre    HTML to prepend to link
 * $post   HTML to append to link
 * $more   Additonal HTML to include into the anchortag
 *
 */

function format_link_wiki(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;
  global $ID; //we use this to get the current namespace
  //obvious setup
  $target = $conf['target']['wiki'];
  $style  = '';
  $pre    = '';
  $post   = '';
  $more   = '';

  //if link contains no namespace. add current namespace (if any)
  if(strpos($ID,':')!==false  && strpos($link,':') === false){
    $ns = substr($ID,0,strrpos($ID,':')); #get current ns
    $link = $ns.':'.$link;
  }

  //use link without namespace as name
  if(empty($name)) $name = preg_replace('/.*:/','',$link);
  $name = htmlspecialchars($name);

  //set class depending on existance
  $link = cleanID($link);
  $file = wikiFN($link);
  if(file_exists($file)){
    $class="wikilink1";
  }else{
    $class="wikilink2";
  }

  //construct the full link
  $link = wl($link);
}

function format_link_externalurl(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;
  //simple setup
  $class  = 'urlextern';
  $target = $conf['target']['extern'];
  $pre    = '';
  $post   = '';
  $style  = '';
  $more   = '';
  $link   = $link; //keep it
  if(!$name) $name = htmlspecialchars($link);
  //thats it :-)
}

//this only works in IE :-(
function format_link_windows(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;
  global $lang;
  //simple setup
  $class  = 'windows';
  $target = $conf['target']['windows'];
  $pre    = '';
  $post   = '';
  $style  = '';
  //Display error on browsers other than IE
  $more = 'onclick="if(document.all == null){alert(\''.htmlspecialchars($lang['nosmblinks'],ENT_QUOTES).'\');}"';

  if(!$name) $name = htmlspecialchars($link);
  $link   = str_replace('\\','/',$link);
  $link   = 'file:///'.$link;
}

function format_link_email(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;
  //simple setup
  $class  = 'mail';
  $target = '';
  $pre    = '';
  $post   = '';
  $style  = '';
  $more   = '';

  $name   = htmlspecialchars($name);
  
  //shields up
  if($conf['mailguard']=='visible'){
    //the mail name gets some visible encoding
    $link = str_replace('@',' [at] ',$link);
    $link = str_replace('.',' [dot] ',$link);
    $link = str_replace('-',' [dash] ',$link);
  }elseif($conf['mailguard']=='hex'){
    for ($x=0; $x < strlen($link); $x++) {
			$encode .= '&#x' . bin2hex($link[$x]).';';
		}
		$link = $encode;
  }
  
  if(!$name) $name = $link;
  $link   = "mailto:$link";
}

function format_link_interwiki(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;

  //obvious ones
  $class  = 'interwiki';
  $target = $conf['target']['interwiki'];
  $pre    = '';
  $post   = '';
  $more   = '';

  //get interwiki short name
  list($wiki,$link) = split('>',$link,2);
  $wiki   = strtolower(trim($wiki)); //always use lowercase
  $link   = trim($link);
  if(!$name) $name = $link;

  //encode special chars
  $name   = htmlspecialchars($name);

  //set default to google
  $url = 'http://www.google.com/search?q=';
  $ico = 'google';

  //load interwikilinks
  //FIXME: loading this once may enhance speed a little bit
  $iwlinks = file('conf/interwiki.conf');

  //add special case 'this'
  $iwlinks[] = 'this '.getBaseURL().'/{NAME}'; 
  
  //go through iwlinks and find URL for wiki
  foreach ($iwlinks as $line){
    $line = preg_replace('/#.*/','',$line); //skip comments
    $line = trim($line);
    list($iw,$iwurl) = preg_split('/\s+/',$line);
    if(!$iw or !$iwurl) continue; //skip broken or empty lines
    //check for match 
    if(strtolower($iw) == $wiki){
      $ico = $wiki;
      $url = $iwurl;
      break;
    }
  }

  //if ico exists set additonal style
  if(file_exists('interwiki/'.$ico.'.png')){
    $style='background: transparent url(interwiki/'.$ico.'.png) 0px 1px no-repeat;';
  }elseif(file_exists('interwiki/'.$ico.'.gif')){
    $style='background: transparent url(interwiki/'.$ico.'.gif) 0px 1px no-repeat;';
  }

  //do we stay at the same server? Use local target
  if( strpos($url,getBaseURL()) === 0 ){
    $target = $conf['target']['wiki'];
  }

  //replace placeholder
  if(strstr($url,'{URL}') !== false){
    $link = str_replace('{URL}',urlencode($link),$url);
  }elseif(strstr($url,'{NAME}') !== false){
    $link = str_replace('{NAME}',$link,$url);
  }else{
    $link = $url.urlencode($link);
  }

  //done :-)
}


function format_link_media(&$link,&$name,&$class,&$target,&$style,&$pre,&$post,&$more){
  global $conf;

  $class  ="media";
  $style  = '';
  $pre    = '';
  $post   = '';
  $more   = '';

  list($name,$title) = split('\|',$name,2);
  $t = htmlspecialchars($title);

  //set alignment from spaces
  if(substr($name,0,1)==' ' && substr($name,-1,1)==' '){
    $pre = '<div align="center">';
    $post = '</div>';
  }elseif(substr($name,0,1)==' '){
    $a = 'align="right"';
  }elseif(substr($name,-1,1)==' '){
    $a = 'align="left"';
  }else{
    $a = 'align="middle"';
  }
  $name = trim($name);
  
  //split into src and parameters
  list($src,$param) = split('\?',$name,2);
  //parse width and height
  if(preg_match('#(\d*)(x(\d*))?#i',$param,$size)){
    if($size[1]) $w = $size[1];
    if($size[3]) $h = $size[3];
  }

  //check for nocache param
  $nocache = preg_match('/nocache/i',$param);
  //do image caching, resizing and src rewriting
  $cache = $src;
  $isimg = img_cache($cache,$src,$w,$h,$nocache);

  //set link to src if none given 
  if(!$link){
    $link = $src;
    $target = $conf['target']['media'];
  }

  //prepare name
  if($isimg){
           $name = '<img src="'.$cache.'"';
    if($w) $name .= ' width="'.$w.'"';
    if($h) $name .= ' height="'.$h.'"';
    if($t) $name .= ' title="'.$t.'"';
    if($a) $name .= $a;
           $name .= ' class="media" border="0" alt="[embedded Image]" />';
  }else{
    $name = basename($src);
  }
}


function img_cache(&$csrc,&$src,&$w,&$h,$nocache){
  global $conf;
  
  //container for various paths
  $f['full']['web'] = $src;
  $f['resz']['web'] = $src;
  $f['full']['fs']  = $src;
  $f['resz']['fs']  = $src;

  //generate cachename
  $md5 = md5($src);

  //check if it is an image
  if(preg_match('#\.(jpe?g|gif|png)$#i',$src,$match)){
    $ext   = strtolower($match[1]);
    $isimg = true;
  }

  //check if it is external or a local mediafile
  if(preg_match('#^([a-z0-9]+?)://#i',$src)){
    $isurl = true;
  }else{
    $src = str_replace(':','/',$src);
    $f['full']['web'] = $conf['mediaweb'].'/'.$src;
    $f['resz']['web'] = $conf['mediaweb'].'/'.$src;
    $f['full']['fs']  = $conf['mediadir'].'/'.$src;
    $f['resz']['fs']  = $conf['mediadir'].'/'.$src;
  }

  //download external images if allowed
  if($isurl && $isimg && !$nocache){
    $cache = $conf['mediadir']."/.cache/$md5.$ext";
    if (file_exists($cache) || download($src,$cache)){
      $f['full']['web'] = $conf['mediaweb']."/.cache/$md5.$ext";
      $f['resz']['web'] = $conf['mediaweb']."/.cache/$md5.$ext";
      $f['full']['fs']  = $conf['mediadir']."/.cache/$md5.$ext";
      $f['resz']['fs']  = $conf['mediadir']."/.cache/$md5.$ext";
      $isurl = false;
    }
  }

  //for local images (cached or media) do resizing
  if($isimg && (!$isurl) && $w){
    $info = getImageSize($f['full']['fs']);
    //if $h not given calcualte it with correct aspect ratio
    if(!$h){
      $h = round(($w * $info[1]) / $info[0]);
    }
    $cache = $conf['mediadir'].'/.cache/'.$md5.'.'.$w.'x'.$h.'.'.$ext;
    //delete outdated cachefile
    if(file_exists($cache) && (filemtime($cache)<filemtime($f['full']['fs']))){
      unlink($cache);
    }
    //check if a resized cachecopy exists else create one
    if(file_exists($cache) || img_resize($ext,$f['full']['fs'],$info[0],$info[1],$cache,$w,$h)){
      $f['resz']['web'] = $conf['mediaweb'].'/.cache/'.$md5.'.'.$w.'x'.$h.'.'.$ext;
      $f['resz']['fs']  = $conf['mediadir'].'/.cache/'.$md5.'.'.$w.'x'.$h.'.'.$ext;
    }
  }

  //set srcs
  $src  = $f['full']['web'];
  $csrc = $f['resz']['web'];
  return $isimg;
}

function img_resize($ext,$from,$from_w,$from_h,$to,$to_w,$to_h){
  // create cachedir
  io_makeFileDir($to);

  // create an image of the given filetype
  if ($ext == 'jpg' || $ext == 'jpeg'){
    if(!function_exists("imagecreatefromjpeg")) return false;
    $image = @imagecreateFromjpeg($from);
  }elseif($ext == 'png') {
    if(!function_exists("imagecreatefrompng")) return false;
    $image = @imagecreatefrompng($from);
  }elseif($ext == 'gif') {
    if(!function_exists("imagecreatefromgif")) return false;
    $image = @imagecreatefromgif($from);
  }
  if(!$image) return false;

  $newimg = imagecreatetruecolor ($to_w, $to_h);

  //try resampling first
  if(function_exists("imagecopyresampled")){
    if(!@imagecopyresampled($newimg, $image, 0, 0, 0, 0, $to_w, $to_h, $from_w, $from_h)) {
      imagecopyresized($newimg, $image, 0, 0, 0, 0, $to_w, $to_h, $from_w, $from_h);
    }
  }else{
    imagecopyresized($newimg, $image, 0, 0, 0, 0, $to_w, $to_h, $from_w, $from_h);
  }
  
  if ($ext == 'jpg' || $ext == 'jpeg'){
    if(!function_exists("imagejpeg")) return false;
    return imagejpeg($newimg, $to, 70);
  }elseif($ext == 'png') {
    if(!function_exists("imagepng")) return false;
    return imagepng($newimg, $to);
  }elseif($ext == 'gif') {
    if(!function_exists("imagegif")) return false;
    return imagegif($newimg, $to);
  }

  return false;
}

?>
